@@ -3,7 +3,9 @@
 import copy
 
 import shutil
+import json
 
+import luigi
 import click
 import pkg_resources
 import yaml
@@ -11,11 +13,12 @@ import logging
 import os
 
 from jinja2 import Template
-import luigi
 from pykwalify.core import Core
 from betterboto import client as betterboto_client
 
-from servicecatalog_puppet.luigi_tasks_and_targets import ProvisionProductTask, SetSSMParamFromProvisionProductTask
+from servicecatalog_puppet import aws
+from servicecatalog_puppet.luigi_tasks_and_targets import ProvisionProductTask, SetSSMParamFromProvisionProductTask, \
+    TerminateProductTask
 from servicecatalog_puppet.commands.list_launches import do_list_launches
 from servicecatalog_puppet.utils import manifest as manifest_utils
 from servicecatalog_puppet.asset_helpers import resolve_from_site_packages, read_from_site_packages
@@ -31,6 +34,15 @@ from servicecatalog_puppet.commands.bootstrap_org_master import do_bootstrap_org
 logger = logging.getLogger()
 logger.setLevel(logging.INFO)
 
+PROVISIONED = 'provisioned'
+TERMINATED = 'terminated'
+
+DISALLOWED_ATTRIBUTES_FOR_TERMINATED_LAUNCHES = [
+    'depends_on',
+    'outputs',
+    'parameters',
+]
+
 
 @click.group()
 @click.option('--info/--no-info', default=False)
@@ -134,6 +146,69 @@ def set_regions_for_deployment_map(deployment_map):
     return deployment_map
 
 
+def get_parameters_for_launch(required_parameters, deployment_map, manifest, launch_details, account_id, status):
+    regular_parameters = []
+    ssm_parameters = []
+
+    for required_parameter_name in required_parameters.keys():
+        account_ssm_param = deployment_map.get(account_id).get('parameters', {}).get(required_parameter_name, {}).get('ssm')
+        account_regular_param = deployment_map.get(account_id).get('parameters', {}).get(required_parameter_name, {}).get('default')
+
+        launch_params = launch_details.get('parameters', {})
+        launch_ssm_param = launch_params.get(required_parameter_name, {}).get('ssm')
+        launch_regular_param = launch_params.get(required_parameter_name, {}).get('default')
+
+        manifest_params = manifest.get('parameters', {})
+        manifest_ssm_param = manifest_params.get(required_parameter_name, {}).get('ssm')
+        manifest_regular_param = manifest_params.get(required_parameter_name, {}).get('default')
+
+        if status == PROVISIONED and account_ssm_param:
+            ssm_parameters.append(
+                get_ssm_config_for_parameter(account_ssm_param, required_parameter_name)
+            )
+        elif status == PROVISIONED and account_regular_param:
+            regular_parameters.append({
+                'name': required_parameter_name,
+                'value': str(account_regular_param),
+            })
+
+        elif launch_ssm_param:
+            ssm_parameters.append(
+                get_ssm_config_for_parameter(launch_ssm_param, required_parameter_name)
+            )
+        elif launch_regular_param:
+            regular_parameters.append({
+                'name': required_parameter_name,
+                'value': launch_regular_param,
+            })
+
+        elif status == PROVISIONED and manifest_ssm_param:
+            ssm_parameters.append(
+                get_ssm_config_for_parameter(manifest_ssm_param, required_parameter_name)
+            )
+        elif status == PROVISIONED and manifest_regular_param:
+            regular_parameters.append({
+                'name': required_parameter_name,
+                'value': manifest_regular_param,
+            })
+
+    return regular_parameters, ssm_parameters
+
+
+def get_ssm_config_for_parameter(account_ssm_param, required_parameter_name):
+    if account_ssm_param.get('region') is not None:
+        return {
+            'name': account_ssm_param.get('name'),
+            'region': account_ssm_param.get('region'),
+            'parameter_name': required_parameter_name,
+        }
+    else:
+        return {
+            'name': account_ssm_param.get('name'),
+            'parameter_name': required_parameter_name,
+        }
+
+
 @cli.command()
 @click.argument('f', type=click.File())
 @click.option('--single-account', default=None)
@@ -149,26 +224,30 @@ def deploy(f, single_account):
     for account_id, deployments_for_account in deployment_map.items():
         for launch_name, launch_details in deployments_for_account.get('launches').items():
             for region_name, regional_details in launch_details.get('regional_details').items():
-                regular_parameters = []
-                ssm_parameters = []
-                for parameter_name, parameter_detail in launch_details.get('parameters', {}).items():
-                    if parameter_detail.get('default') is not None:
-                        regular_parameters.append({
-                            'name': parameter_name,
-                            'value': parameter_detail.get('default')
-                        })
-                    if parameter_detail.get('ssm') is not None:
-                        if parameter_detail.get('ssm').get('region') is not None:
-                            ssm_parameters.append({
-                                'name': parameter_detail.get('ssm').get('name'),
-                                'region': parameter_detail.get('ssm').get('region'),
-                                'parameter_name': parameter_name,
-                            })
-                        else:
-                            ssm_parameters.append({
-                                'name': parameter_detail.get('ssm').get('name'),
-                                'parameter_name': parameter_name,
-                            })
+                product_id = regional_details.get('product_id')
+                required_parameters = {}
+
+                role = f"arn:aws:iam::{account_id}:role/servicecatalog-puppet/PuppetRole"
+                with betterboto_client.CrossAccountClientContextManager(
+                        'servicecatalog', role, f'sc-{account_id}-{region_name}', region_name=region_name
+                ) as service_catalog:
+                    response = service_catalog.describe_provisioning_parameters(
+                        ProductId=product_id,
+                        ProvisioningArtifactId=regional_details.get('version_id'),
+                        PathId=aws.get_path_for_product(service_catalog, product_id),
+                    )
+                    for provisioning_artifact_parameters in response.get('ProvisioningArtifactParameters', []):
+                        parameter_key = provisioning_artifact_parameters.get('ParameterKey')
+                        required_parameters[parameter_key] = True
+
+                    regular_parameters, ssm_parameters = get_parameters_for_launch(
+                        required_parameters,
+                        deployment_map,
+                        manifest,
+                        launch_details,
+                        account_id,
+                        launch_details.get('status', PROVISIONED),
+                    )
 
                 logger.info(f"Found a new launch: {launch_name}")
 
@@ -190,6 +269,8 @@ def deploy(f, single_account):
 
                     'depends_on': launch_details.get('depends_on', []),
 
+                    "status": launch_details.get('status', PROVISIONED),
+
                     'dependencies': [],
                 }
 
@@ -210,13 +291,33 @@ def deploy(f, single_account):
 
                 all_tasks[f"{task.get('account_id')}-{task.get('region')}-{task.get('launch_name')}"] = task
 
-    for task_uid, task in all_tasks.items():
-        for dependency in task.get('depends_on', []):
-            for task_uid_2, task_2 in all_tasks.items():
-                if task_2.get('launch_name') == dependency:
-                    task.get('dependencies').append(task_2)
-        del task['depends_on']
-        tasks_to_run.append(ProvisionProductTask(**task))
+    logger.info(f"Deployment plan: {json.dumps(all_tasks)}")
+
+    for task in wire_dependencies(all_tasks):
+        task_status = task.get('status')
+        del task['status']
+        if task_status == PROVISIONED:
+            tasks_to_run.append(ProvisionProductTask(**task))
+        elif task_status == TERMINATED:
+            for attribute in DISALLOWED_ATTRIBUTES_FOR_TERMINATED_LAUNCHES:
+                logger.info(f"checking {launch_name} for disallowed attributes")
+                attribute_value = task.get(attribute)
+                if attribute_value is not None:
+                    if isinstance(attribute_value, list):
+                        if len(attribute_value) != 0:
+                            raise Exception(f"Launch {task.get('launch_name')} has disallowed attribute: {attribute}")
+                    elif isinstance(attribute_value, dict):
+                        if len(attribute_value.keys()) != 0:
+                            raise Exception(f"Launch {task.get('launch_name')} has disallowed attribute: {attribute}")
+                    else:
+                        raise Exception(f"Launch {task.get('launch_name')} has disallowed attribute: {attribute}")
+
+            for a in ['parameters', 'ssm_param_inputs', 'outputs', 'dependencies']:
+                if task.get(a, None) is not None:
+                    del task[a]
+            tasks_to_run.append(TerminateProductTask(**task))
+        else:
+            raise Exception(f"Unsupported status of {task_status}")
 
     luigi.build(
         tasks_to_run,
@@ -227,6 +328,19 @@ def deploy(f, single_account):
     )
 
 
+def wire_dependencies(all_tasks):
+    tasks_to_run = []
+    for task_uid, task in all_tasks.items():
+        for dependency in task.get('depends_on', []):
+            for task_uid_2, task_2 in all_tasks.items():
+                if task_2.get('launch_name') == dependency:
+                    task.get('dependencies').append(task_2)
+        del task['depends_on']
+        logger.info(f"Scheduling ProvisionProductTask: {json.dumps(task)}")
+        tasks_to_run.append(task)
+    return tasks_to_run
+
+
 @cli.command()
 @click.argument('puppet_account_id')
 @click.argument('iam_role_arns', nargs=-1)
